/* ====================================================================
 *
 * � 2010 Infosys Technologies Limited, Bangalore, India. All rights reserved. 
 * Infosys Test Automation Accelerator * Version: 6.7

 * Except for any open source software components embedded in this Infosys proprietary software program 
 * ("Program"), this Program is protected by copyright laws, international treaties and other pending or existing intellectual
 * property rights in India, the United States 
 * and other countries. Except as expressly permitted, any unauthorized reproduction, storage, transmission in any form 
 * or by any means (including without limitation electronic, mechanical, printing, photocopying, recording or otherwise),
 * or any distribution of this Program, or any portion of it, may result in severe civil and criminal penalties, and will be prosecuted to the maximum extent possible under the law.

 ======================================================================*/
/*****************************************************************************
 *   Revision History
 *   ------------------
 *   Name         :  LoginDAO.java
 *   Author       :  Kirty_vs
 *   Date         :  10/August/2007
 *   Description  :  This class represents Data Access object for Login.
 *
 *   Modification Log
 * 	 Date                 Author                       Description
 * 	 ------------------------------------------------------------
 * 	 10/August/2007 	Kirty_vs 	          Created
 *   17-Jun-2008        Chitra_Subramanian    Added 2 new functions strSessionId
 *   and setSessionId for performance tuning of ITAW
 **************************************************************************/

package com.infosys.itaw.login.persistence.dao;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.hibernate.Query;
import org.springframework.stereotype.Repository;

import com.infosys.itaw.user.persistence.dto.UserDTO;
import com.infosys.itaw.util.GenericHibernateDAO;

import itawexception.ItawException;
//import com.infosys.j2ee.core.ServiceException;
//import com.infosys.j2ee.core.impl.RadienServiceLocator;
//import com.infosys.j2ee.infrastructure.service.logging.LoggingService;
//import com.infosys.j2ee.subframework.persistence.dao.DBException;
//import com.infosys.j2ee.subframework.persistence.dao.impl.GenericHibernateDAO;

/**
 * This is the DAO class for Login Module.
 * 
 * @author Kirty_vs
 */
@Repository
public class LoginDAO extends GenericHibernateDAO {

	UserDTO userDto = null;
	int iRoleId = 0;
	//LoggingService log = null;

	// Session session=null;

	public LoginDAO() throws ItawException {
		try {
			//log = (LoggingService) RadienServiceLocator.getInstance().getService("default-Log4jLoggingService");
	} 
			//catch (ServiceException e) {
//			throw new ItawException("ITAW11013");
//		} 
		catch (Exception e) {
			// System.out.println("Service initialization error");
			throw new ItawException("ITAW11014");
		}
	}

	/**
	 * Check if any user with the corresponding password exist or not
	 * 
	 * @param strUserName-name
	 *            of the user strPassword-password of the user
	 * @return iUserId-user id for success and 0 for failure
	 * @throws ItawException
	 */
	public int intCheckUser(String strUserName, String strPassword) throws ItawException {
		int iUserId = 0;
		try {
			UserDTO userDTO = this.getUser(strUserName);
			if (userDTO != null) {
				UserDTO userDTO2 = this.validateUserPassword(strUserName, strPassword);
				boolean isValidated = userDTO2 != null;
				//iUserId = this.updateFailedLoginDetails(isValidated ? userDTO2 : userDTO, isValidated);
				iUserId=userDTO2.getiUserId();
			}
		}
		// catch (ServiceException serviceException){
		// service//exception.printStackTrace();
		// throw new ItawException("ITAW2501");
		// }
//		catch (DBException dbException) {
//
//			// db//exception.printStackTrace();
//			throw new ItawException("ITAW2502");
//		} 
		catch (Exception exception) {
			exception.printStackTrace();
			throw new ItawException("ITAW2503");
		}

		// clearing all the session variables
		// session.clear();
		return iUserId;
	}

	/**
	 * Retrive the privileges associated with the user
	 * 
	 * @param iUserId-Id
	 *            of the user
	 * @return arrPrivilege-arraylist of privileges associated with user
	 * @throws ItawException
	 */
	public ArrayList arrGetPrivileges(int iUserId) throws ItawException {

		ArrayList arrPrivilege = new ArrayList();
		String strQuery = "SELECT DISTINCT p.IPrivilegeId,p.strObject,p.COperation from RolePrivilegesDTO as rp,PrivilegeDTO as p where rp.IPrivilegeId = p.IPrivilegeId";
		strQuery = strQuery + " and rp.IRoleId IN (SELECT  ur.IRoleId from UserRoleDTO as ur,RoleDTO as r where ur.IRoleId = r.IRoleId and r.IEnableStatus=1 and ur.IUserId=" + iUserId + ")";

		try {
			// getting current session
			// session=HibernateSessionUtil.currentSession("testfactory");

			// Create an instance of Logging Service of Radien Service Locator
			// log=(LoggingService)
			// RadienServiceLocator.getInstance().getService("default-Log4jLoggingService");
			//log.info("ITAW-LoginDAO", "Entered method arrGetPrivileges of DAO class");

			// Execute HQL Query to return arraylist of privileges associated
			// with the user in the database
			//arrPrivilege = (ArrayList) executeHQLQuery(strQuery);

		}
		// catch (ServiceException serviceException){
		// service//exception.printStackTrace();
		// throw new ItawException("ITAW2504");
		// }
//		catch (DBException dbException) {
//			// db//exception.printStackTrace();
//			throw new ItawException("ITAW2505");
//		} 
		catch (Exception exception) {
			// exception.printStackTrace();
			throw new ItawException("ITAW2506");
		}

		// clearing all the session variables
		// session.clear();
		return arrPrivilege;
	}

	/**
	 * Updates new password on forget password
	 * 
	 * @param strUserName-Name
	 *            of user strNewPassword-New password generated
	 * @return strEmail-email id for success and empty string for failure
	 * @throws ItawException
	 */
	public String strForgetPassword(String strUserName, String strNewPassword) throws ItawException {

		List lstResult1 = null;
		String strEmail = "";
		String strQuery = null;
		Query query = null;
		ArrayList user = new ArrayList();
		

		try {
			// getting current session
			// session=HibernateSessionUtil.currentSession("testfactory");
			// Transaction transaction=(Transaction)getTransaction();

			// Create an instance of Logging Service of Radien Service Locator
			// log=(LoggingService)
			// RadienServiceLocator.getInstance().getService("default-Log4jLoggingService");

			// Execute HQL query for checking if the user exist
			strQuery = this.getSQL("SELECT_BY_NAME");
			query = this.createQuery(strQuery);
			// Retrieve all the rows with name as strUserName
			
			 user = (ArrayList) query.setString(0, strUserName).list();

			System.out.println("ITAW-LoginDAO Size:" + user.size());

			// check if the user exist
			if (0 < user.size()) {
				userDto = (UserDTO) lstResult1.get(0);
				int iUserId = userDto.getiUserId();
				System.out.println("ITAW-LoginDAO UserId:" + iUserId);
				Timestamp date1 = new Timestamp(new Date().getTime());
				// set all the values in user pojo for update
				userDto.setiUserId(iUserId);
				userDto.setStrUserName(strUserName);
				userDto.setStrPassword(strNewPassword);
				userDto.setDateCreation(date1);
				strEmail = userDto.getStrEmailId();
				// transaction.begin();

				// execute update
				update(userDto);

				// commiting the transaction
				// getTransaction().commit();

			} else {
				System.out.println("ITAW-LoginDAO"+ "User doesnt exist");
			}

		}
		// catch (ServiceException serviceException){
		//
		// service//exception.printStackTrace();
		// throw new ItawException("ITAW2507");
		// }
//		catch (DBException dbException) {
//			// db//exception.printStackTrace();
//			throw new ItawException("ITAW2508");
//		}
		/*
		 * catch (HibernateDBException hibernate) { hibernate.printStackTrace();
		 * throw new ItawException("ITAW2509"); }
		 */
		catch (Exception excception) {
			// excception.printStackTrace();
			throw new ItawException("ITAW2510");
		}

		// clearing all the session variables
		// session.clear();
		return strEmail;
	}

	/**
	 * Retrive the projects of a particular user
	 * 
	 * @param iUserId-Id
	 *            of the user tool for the tool list searchFlag for determining
	 *            if a search or normal load searchName to search against the
	 *            name entered by user
	 * @return arrProject-arraylist of projects of a particular user
	 * @throws ItawException
	 */
	// public ArrayList arrLstProject(int iUserId,ArrayList tool)throws
	// ItawException{ //Top-Down approach
/*	public ArrayList arrLstProject(int iUserId, ArrayList tool, int searchFlag, String searchName) throws ItawException { // Top-Down
		// approach
		ArrayList arrProject = new ArrayList();

		try {
			// String [] tool = {"WinRunner","Quick Test Pro"};
			// getting current session
			// session=HibernateSessionUtil.currentSession("testfactory");

			// Create an instance of Logging Service of Radien Service Locator
			// log=(LoggingService)
			// RadienServiceLocator.getInstance().getService("default-Log4jLoggingService");
			//log.info("ITAW-LoginDAO", "Entered method arrLstProject of DAO class");

			// arrProject=(ArrayList)executeHQLQuery("select
			// pu.IProjectId,p.strProjectName,p.strProjectDesc,p.strProjectCode,p.strManager,p.strManagerEmail,p.strProjectToolType
			// from ProjectUsersDTO as pu,ProjectDTO as p where pu.IUserId
			// ="+iUserId+ "and pu.IProjectId=p.IProjectId");
			String query = "select pu.IProjectId,p.strProjectName,p.strProjectDesc,p.strProjectCode,p.strManager,p.strManagerEmail,p.strProjectToolType,p.strProjectToolVersion,p.strBddType from ProjectUsersDTO as pu,ProjectDTO as p where pu.IUserId =" + iUserId + " and pu.IProjectId=p.IProjectId and (";
			int iToolSize = tool.size();
			for (int i = 0; i < iToolSize; i++) {
				query += "p.strProjectToolType='" + tool.get(i) + "'";
				if (i != tool.size() - 1)
					query += " OR ";
			}
			// Top-Down approach changes begin
			// query+=")";
			if (searchFlag == 0) {
				query += ") order by pu.IProjectId desc";
			} else {
				query += ") and p.strProjectName like '" + searchName + "%' order by pu.IProjectId desc";
			}
			// Top-Down approach changes ends
			// Execute HQL Query to return arraylist of projects of a particular
			// user in the database
			arrProject = (ArrayList) executeHQLQuery(query);
			// log.info("ITAW-LoginDAO","Value of
			// toolversion"+arrProject.get(7));
			//log.info("ITAW-LoginDAO", "Value of toolversion" + arrProject.size());
			System.out.println("query is " + query);
			System.out.println("arrproj: " + arrProject.size());
		}
		// catch (ServiceException serviceException){
		//
		// service//exception.printStackTrace();
		// throw new ItawException("ITAW2524");
		// }
//		catch (DBException dbException) {
//			// db//exception.printStackTrace();
//			throw new ItawException("ITAW2525");
//		} 
		catch (Exception exception) {
			// exception.printStackTrace();
			throw new ItawException("ITAW2526");
		}

		// clearing all the session variables
		// session.clear();
		return arrProject;
	}

	public int iprojectType(int projectId, String userName) throws ItawException {
		int projectType = 0;
		List lstResult = null;

		ProjectDTO pType = new ProjectDTO();
		ProjectSessionDTO Projectsession = new ProjectSessionDTO();
		try {
			// getting current session
			// session=HibernateSessionUtil.currentSession("testfactory");

			System.out.println("UseriD:::::::::" + userName);

			// Delete from project session
			String strDeleteRule = this.getSQL("DELETE_PROJECTID");
			System.out.println("String" + strDeleteRule);
			Query query = (Query) this.createQuery(strDeleteRule);
			query.setString(0, userName).executeUpdate();

			// creating session for selected project
			Projectsession.setIProjectId(projectId);
			Projectsession.setStrUsername(userName);
			create(Projectsession);
			getTransaction().commit();

			// Create an instance of Logging Service of Radien Service Locator
			// log=(LoggingService)
			// RadienServiceLocator.getInstance().getService("default-Log4jLoggingService");
			//log.info("ITAW-LoginDAO", "Entered method iprojectType of DAO class");
			lstResult = (List) executeHQLQuery("from ProjectDTO where IProjectId =" + projectId);

			pType = (ProjectDTO) lstResult.get(0);

			projectType = pType.getIProjectType();

		}
		// catch (ServiceException serviceException){
		// service//exception.printStackTrace();
		// throw new ItawException("ITAW2524");
		// }
//		catch (DBException dbException) {
//			// db//exception.printStackTrace();
//			throw new ItawException("ITAW12619");
//		}
		catch (Exception exception) {
			// exception.printStackTrace();
			throw new ItawException("ITAW12621");
		}

		// clearing all the session variables
		// session.clear();
		return projectType;
	}*/

	/**
	 * Returns the session id corresponding to the user id.New function added
	 * for Performance tuning.
	 * 
	 * @param userId-Id
	 *            of User
	 * @return sessionId
	 * @throws ItawException
	 *             Modification Log Date Author Description
	 *             ----------------------------------------------------------------------
	 *             17-Jun-2008 Chitra_Subramanian Added a new function
	 *             strsessionId for performance tuning of ITAW
	 */
	/*public String strsessionId(int userId) throws ItawException {

		List lstResult = null;
		String strSessionId;

		SessionDetailsDTO sessionDetails = new SessionDetailsDTO();

		try {
			// getting current session
			// session=HibernateSessionUtil.currentSession("testfactory");

			// Create an instance of Logging Service of Radien Service Locator
			// log=(LoggingService)
			// RadienServiceLocator.getInstance().getService("default-Log4jLoggingService");
			//log.info("ITAW-LoginDAO", "Entered method strsessionId of DAO class");
			lstResult = (List) executeHQLQuery("from SessionDetailsDTO where IUserId =" + userId);

			sessionDetails = (SessionDetailsDTO) lstResult.get(0);

			strSessionId = sessionDetails.getStrSessionId();

		}
		// catch (ServiceException serviceException){
		//
		// service//exception.printStackTrace();
		// throw new ItawException("ITAW2524");
		// }
//		catch (DBException dbException) {
//			// db//exception.printStackTrace();
//			throw new ItawException("ITAW12620");
//		} 
		catch (Exception exception) {
			// exception.printStackTrace();
			throw new ItawException("ITAW12622");
		}

		// clearing all the session variables
		// session.clear();
		return strSessionId;
	}*/

	/**
	 * Sets the session id corresponding to the user id.New function added for
	 * Performance tuning.
	 * 
	 * @param strUserId-Id
	 *            of User
	 * @param strSessionId-Session
	 *            Id
	 * @return iResult int
	 * @throws ItawException
	 * @Modification Log Date Author Description
	 *               ----------------------------------------------------------------------
	 *               17-Jun-2008 Chitra_Subramanian Added a new function
	 *               setsessionId for performance tuning of ITAW
	 */
	/*public int setsessionId(int userId, String strSessionId) throws ItawException {

		List lstResult = null;

		SessionDetailsDTO sessiondetailsdto;
		int iResult = 0;
		int iSize = 0;

		try {
			// getting current session
			// session=HibernateSessionUtil.currentSession("testfactory");
			// Transaction transaction=(Transaction)getTransaction();

			// Create an instance of Logging Service of Radien Service Locator
			// log=(LoggingService)
			// RadienServiceLocator.getInstance().getService("default-Log4jLoggingService");

			// Execute HQL query for checking if the user exist
			lstResult = (List) executeHQLQuery("from SessionDetailsDTO where IUserId=" + userId);
			iSize = lstResult.size();
			//log.info("ITAW-LoginDAO", "Size:" + iSize);

			// check if the user exist
			if (0 < iSize) {
				
				 * sessiondetailsdto=(SessionDetailsDTO)lstResult.get(0); int
				 * iUserId=sessiondetailsdto.getiUserId();
				 * log.info("ITAW-LoginDAO","UserId:"+iUserId);
				 
				//log.info("ITAW-LoginDAO", "SessionId:" + strSessionId);

				// set all the values in user pojo for update
				// sessiondetailsdto.setIUserId(iUserId);
				// sessiondetailsdto.setStrSessionId(strSessionId);
				// transaction.begin();

				String query1 = "UPDATE SessionDetailsDTO SET strSessionId = '" + strSessionId + "' WHERE IUserId=" + userId;

				Query selQuery = (Query) this.createQuery(query1);
				iResult = selQuery.executeUpdate();

				// execute update
				// update(sessiondetailsdto);

				// commiting the transaction
				// getTransaction().commit();

			} else {
				//log.info("ITAW-LoginDAO", "User doesnt exist");
			}

		}
		// catch (ServiceException serviceException){
		//
		// service//exception.printStackTrace();
		// throw new ItawException("ITAW2507");
		// }
//		catch (DBException dbException) {
//			// db//exception.printStackTrace();
//			throw new ItawException("ITAW12617");
//		}
		
		 * catch (HibernateDBException hibernate) { hibernate.printStackTrace();
		 * throw new ItawException("ITAW2509"); }
		 
		catch (Exception excception) {
			// excception.printStackTrace();
			throw new ItawException("ITAW12618");
		}

		// clearing all the session variables
		// session.clear();
		return iResult;
	}*/

	/**
	 * 
	 */
/*	public ArrayList getPlatformandToolList() throws ItawException {
		List lstAppPlatform = null;
		List lstTools = null;
		ArrayList arrList = new ArrayList();
		try {
			lstAppPlatform = (ArrayList) executeHQLQuery("Select Distinct strAppPlatformName from AppPlatformDTO");
			lstTools = (ArrayList) executeHQLQuery("Select Distinct strTool from FunctionDTO");
			System.out.println("Size" + lstAppPlatform.size());
			arrList.add(lstAppPlatform.size());

			for (int i = 0; i < lstAppPlatform.size(); i++) {
				arrList.add(lstAppPlatform.get(i));
			}
			for (int i = 0; i < lstTools.size(); i++) {
				arrList.add(lstTools.get(i));
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			throw new ItawException("ITAW12618");
		}
		return arrList;
	}*/

	// for SSO
	public int intCheckUser(String strUserName) throws ItawException {
		int iUserId = 0;
		try {
			UserDTO userDTO = this.getUser(strUserName);
			if (userDTO != null) {
				iUserId = this.updateFailedLoginDetails(userDTO, true);
			}
		}
//		catch (DBException dbException) {
//			throw new ItawException("ITAW2502");
//		}
		catch (Exception exception) {
			throw new ItawException("ITAW2503");
		}
		return iUserId;
	}

	private UserDTO getUser(String strUserName) throws Exception {
		// Execute HQL Query to return a list when the user and
		// corresponding user exist in the database

		
	
		String strQuery = null;
		Query query = null;
		ArrayList user = new ArrayList();
		

		/*try {*/
			// getting current session
			// session=HibernateSessionUtil.currentSession("testfactory");
			// Transaction transaction=(Transaction)getTransaction();

			// Create an instance of Logging Service of Radien Service Locator
			// log=(LoggingService)
			// RadienServiceLocator.getInstance().getService("default-Log4jLoggingService");

			// Execute HQL query for checking if the user exist
			strQuery = this.getSQL("SELECT_BY_NAME");
			query = this.createQuery(strQuery);
			// Retrieve all the rows with name as strUserName
			
			 user = (ArrayList) query.setString(0, strUserName).list();
	
		//log.info("ITAW-LoginDAO", lstResult1);
		return (user.size() > 0) ? (UserDTO) user.get(0) : null;
	}

	private UserDTO validateUserPassword(String strUserName, String strPassword) throws Exception {
		String strQuery = null;
		Query query = null;
		ArrayList user = new ArrayList();
		strQuery = this.getSQL("SELECT_BY_PASSWORD");
		query = this.createQuery(strQuery);
		// Retrieve all the rows with name as strUserName
		
		 user = (ArrayList) query.setString(0, strUserName).setString(1, strPassword).list();

	//log.info("ITAW-LoginDAO", lstResult1);
	return (user.size() > 0) ? (UserDTO) user.get(0) : null;
	}

	private int updateFailedLoginDetails(UserDTO userDTO, boolean isValidated) throws Exception {
		int iUserId = 0;
		int itemp;
		Date dateCreattion;
		Date endDate;
		Date dateFailedLogin;
		int diffInDays = 0;
		boolean isUserExpired = false;

		// get the user id
		if (isValidated) {

			iUserId = userDTO.getiUserId();
			dateFailedLogin = userDTO.getDateFailedLogin();
			if (dateFailedLogin != null) {
				Date currentDate = new Date();
				diffInDays = (int) ((currentDate.getTime() - dateFailedLogin.getTime()) / (1000 * 60 * 60 * 24));
				// unlock account by reseting values to default values,
				// if account was locked one day back
				if (diffInDays >= 1 || userDTO.getFailedLoginAttempts() < 3) {
					String query = this.getSQL("UPDATE_dateFailedLogin");
					Query query1 = (Query) this.createQuery(query);
					query1.setInteger(0, 0).setDate(1, null).setInteger(2, userDTO.getiUserId()).executeUpdate();
					getTransaction().commit();
				} else {
					iUserId = -3;
				}
			}
			// code for temporary User
			itemp = userDTO.getIntTemp();
			dateCreattion = userDTO.getDateCreation();
			endDate = userDTO.getEndDate();
			Date date1 = new Date();
			diffInDays = (int) ((date1.getTime() - dateCreattion.getTime()) / (1000 * 60 * 60 * 24));
			// diffInDays = (int)( (endDate.getTime() -
			// dateCreattion.getTime())/ (1000 * 60 * 60 * 24) );
			if (endDate != null)
				isUserExpired = date1.after(endDate);

			if (itemp == 1) {
				if (isUserExpired) {
					iUserId = -4;
				}
			} else {// Password Expired after 60days.
				if (diffInDays > 60) {
					iUserId = -5;
				}
			}
		} else {
			if (userDTO.getDateFailedLogin() == null) {
				String query = this.getSQL("UPDATE_dateFailedLogin");
				Query query1 = (Query) this.createQuery(query);
				query1.setInteger(0, 1).setDate(1, new Date()).setInteger(2, userDTO.getiUserId()).executeUpdate();
				getTransaction().commit();
				iUserId = -1;
			} else {
				if ((int) ((new Date().getTime() - userDTO.getDateFailedLogin().getTime()) / (1000 * 60 * 60 * 24)) >= 1) {
					String query = this.getSQL("UPDATE_dateFailedLogin");
					Query query1 = (Query) this.createQuery(query);
					query1.setInteger(0, 1).setDate(1, new Date()).setInteger(2, userDTO.getiUserId()).executeUpdate();
					getTransaction().commit();
					iUserId = -1;
				} else {
					int currentNoOfFailedAttempts = userDTO.getFailedLoginAttempts();
					currentNoOfFailedAttempts++;
					dateFailedLogin = new Date();

					if (currentNoOfFailedAttempts > 3) {
						iUserId = -3;
					} else if (currentNoOfFailedAttempts == 3) {
						String query = this.getSQL("UPDATE_NoOfFailedAttempts");
						Query query1 = (Query) this.createQuery(query);
						query1.setInteger(0, currentNoOfFailedAttempts).setDate(1, dateFailedLogin).setInteger(2, userDTO.getiUserId()).executeUpdate();
						getTransaction().commit();
						iUserId = -3;
					} else if (currentNoOfFailedAttempts == 2) {
						String query = this.getSQL("UPDATE_NoOfFailedAttempts");
						Query query1 = (Query) this.createQuery(query);
						query1.setInteger(0, currentNoOfFailedAttempts).setDate(1, dateFailedLogin).setInteger(2, userDTO.getiUserId()).executeUpdate();
						getTransaction().commit();
						iUserId = -2;
					}
				}
			}
		}
		return iUserId;
	}
}
