/* ====================================================================
 *
 * � 2010 Infosys Technologies Limited, Bangalore, India. All rights reserved. 
 * Infosys Test Automation Accelerator * Version: 6.7

 * Except for any open source software components embedded in this Infosys proprietary software program 
 * ("Program"), this Program is protected by copyright laws, international treaties and other pending or existing intellectual
 * property rights in India, the United States 
 * and other countries. Except as expressly permitted, any unauthorized reproduction, storage, transmission in any form 
 * or by any means (including without limitation electronic, mechanical, printing, photocopying, recording or otherwise),
 * or any distribution of this Program, or any portion of it, may result in severe civil and criminal penalties, and will be prosecuted to the maximum extent possible under the law.

 ======================================================================*/
package com.infosys.itaw.role.business.service;

/*Importing java classes*/
import java.util.List;

import org.springframework.stereotype.Component;

import com.infosys.itaw.role.persistence.dto.RoleDTO;

/*Importing ITAW classes*/
import itawexception.ItawException;


@Component
public interface RoleService
{
	/**
	 * fetches all the roles
	 * @return list of roles
	 */
	public List listRoles() throws ItawException;
	
	/**
	 * Creates a role and returns 1 on success
	 * @param roleVO
	 * @return 1 on success
	 * @throws 
	 */
	public int createRole(RoleDTO roleVO) throws ItawException;
//	public boolean checkRoleExists(String roleName)throws ItawException;;
	
	/**
	 * Deletes a role and return 1 if deletion is successful
	 * @param iRoleId
	 * @return 1 on success
	 * @throws 
	 */
	public int deleteRoles(int iRoleId)throws ItawException;
	
	/**
	 * Modifies a role and returns 1 on success
	 * @param roleVO
	 * @param iRoleId
	 * @return 1 on success
	 * @throws 
	 */
	public int modifyRole(RoleDTO roleVO,int iRoleId)throws ItawException;
	
	/**
	 * fetch all the role details
	 * @return list of role details
	 */
	public List listRoleDetails(int iRoleId) throws ItawException;
	
	/**
	 * fetch all the privileges
	 * @return list of privileges
	 */
	public List listAllPrivileges() throws ItawException;
	
	/**
	 * fetch all the tree privileges
	 * @return list of tree privileges
	 */
	public List listTreePrivileges() throws ItawException;
	
	/**
	 * fetch all the privileges details
	 * @return list of privileges details
	 */
	public List listPrivilegeDetails(int iRoleId) throws ItawException;
	
	/**
	 * associates a privilege to a role 
	 * @param roleId,newPrivilegeIds
	 * @return 1 on success
	 * @throws 
	 */
	public int associateNewRolePrivilege(int newPrivilegeIds,int roleId) throws ItawException;
	
	/**
	 * disassociates a privilege from a role 
	 * @param roleId,PrivilegeId
	 * @return 1 on success
	 * @throws 
	 */
	public int disassociateRolePrivilege(int privilegeId,int roleId) throws ItawException;
	
	/**
	 * Searches for a privilege from the tree 
	 * @param strPrivilegeName
	 * @return List
	 * @throws 
	 */
	public List searchPrivileges(String strPrivilegeName) throws ItawException;
	
	public List listRolesForCreate() throws ItawException;
}
