
/*****************************************************************************
 *   Revision History
 *   ------------------
 *   Name         :  UserServiceImpl.java
 *   Author       :  Neha_mittal03
 *   Date         :  20/August/2007
 *   Description  :  This class implements the services declared  in  interface UserService.java
 *
 *   $Log		  : UserServiceImpl.java$
 *   Modification Log:
 *	 * Date                 Author                     Description
 *	 * ------------------------------------------------------------
 *	 * 20-Aug-2007          Neha_mittal03              Created
 *	 * 16-Jun-2008 			Abraham_Mammen			   Created function addSessionDetails for performance tuning 
 *   * 23-Jul-2008			Suja_Pillai				   Added a new parameter for multiple association of roles during User creation in createUser function
 *   * 13-Jul-2009			Marian_Josephus	 		   Added rollback and transaction begin code to all functions with commit.
 *	 #  
 *	 #  
 *****************************************************************************/
package com.infosys.itaw.role.business.service.impl;

import java.util.List;

import org.hibernate.HibernateException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.infosys.itaw.role.business.service.RoleService;
import com.infosys.itaw.role.persistence.dao.RoleDAO;
import com.infosys.itaw.role.persistence.dto.RoleDTO;

import itawexception.ItawException;
//import util.UtilFunctions; /*


/**
 * This is the service implementation class for Manage User Module.
 * 
 * @author Neha_mittal03
 */
@Service("RoleService")
@Transactional
public class RoleServiceImpl implements RoleService {
	
	@Autowired
	RoleDAO roleDAO;
	//DAOService hibernateDAOService;
	//LoggingService log;
	// Session session=null;

	public RoleServiceImpl() throws ItawException {
		super();
		try {
		//	hibernateDAOService = (DAOService) RadienServiceLocator.getInstance().getService("default-HibernateDAOService");
		//	userdao = (UserDAO) hibernateDAOService.getDAO("UserDAO");
		
			// session=HibernateSessionUtil.currentSession("testfactory");

		} 
		/*catch (DAOServiceException e) {
			throw new ItawException("ITAW11175");
		}*/
		/*
		 * catch (HibernateException e) { throw new ItawException("ITAW11176"); }
		 */
		catch (Exception e) {
			// System.out.println("Service Initialization error");
			throw new ItawException("ITAW11089");
		}
	}

	/**
	 * fetch all the roles
	 * @return list of roles
	 */
	public List listRoles() throws ItawException 
	{ 
		// lstroles is used to store list of roles
		List lstroles=null;
		try
		{
			//roleDAO = (roleDAO)hibernateDAOService.getDAO("roleDAO");
			//invoking the DAO method and gets the returned value
			lstroles = roleDAO.listRoles();
		}
		catch(ItawException ItawException)
		{
			throw ItawException;
		}
		/*catch (DAOServiceException daoException)
		{
			throw new ItawException("ITAW1024");
		}*/
		catch(Exception exception)
		{
			throw new ItawException("ITAW1025");
			
		}finally{
			try{
				
				roleDAO.closeSession();
					
				
			}
			catch(HibernateException e){
				throw new ItawException("ITAW11356");
			}
		}
		return lstroles;
	}
	
	
	
	/**
	 * Creates a role and return 1 on success
	 * @param roleVO
	 * @return 1 on success
	 * @throws 
	 */
	public int createRole(RoleDTO roleVO)throws ItawException 
	{
		//iResult is used to store the return value
		int iResult=0;
		try
		{
			//Begining transaction
			roleDAO.getSession();
			//roleDAO = (roleDAO)hibernateDAOService.getDAO("roleDAO");
			System.out.println("RoleServiceImpl : createRole " + roleVO.getStrRoleName());
			System.out.println("RoleCreation  "+ roleVO.getStrRoleDesc());
			//invoking the DAO method and stores the return value  
		    iResult=roleDAO.createRole(roleVO);
//		    System.out.println("RoleServiceImpl : createRole","Inserted Successfully");
//		    System.out.println("RoleServiceImpl : createRole","ROLE EXISTS");
			if(2 == iResult)
			{
				System.out.println("RoleServiceImpl : createRole"+"ROLE EXISTS"+roleVO.getIRoleId());
			}
		    else if(1 == iResult)
		    {
		    	System.out.println("RoleServiceImpl : createRole"+ "Insert Successful "+roleVO.getIRoleId());
		    }
		}
		catch(ItawException ItawException)
		{
			//Rollback code
			  try{				
					roleDAO.getTransaction().rollback();	
					}
					catch(HibernateException dbe){
					throw new ItawException("ITAW12501");
					}
//					catch(HibernateException dbe){
//					throw new ItawException("ITAW12502");
//				    }
			throw ItawException;
		}
		/*catch (DAOServiceException daoException)
		{
			throw new ItawException("ITAW1026");
		}*/
		catch(Exception exception)
		{
			//Rollback code
			  try{				
				roleDAO.getTransaction().rollback();	
				}
				catch(HibernateException dbe){
				throw new ItawException("ITAW12503");
				}
//				catch(HibernateException dbe){
//				throw new ItawException("ITAW12504");
//			    }
			throw new ItawException("ITAW1027");
			
		}finally{
			try{
				
				roleDAO.closeSession();
					
				
			}catch(HibernateException dbe){
				throw new ItawException("ITAW11357");
			}
		}
		return iResult;
	}
	
	/**
	 * Deletes roles and return 1 if deletion is success
	 * @param iRoleId
	 * @return 1 on success
	 * @throws 
	 */
	public int deleteRoles(int iRoleId)throws ItawException
	{
		//iResult is used to store the return value
		int iResult=0;	
		try{
			//Begining transaction
			roleDAO.getSession().beginTransaction();
			System.out.println("RoleServiceImpl : deleteRoles"+" entered");
			System.out.println("RoleServiceImpl : deleteRoles"+" before roleDAO is created");
			//roleDAO = (roleDAO)hibernateDAOService.getDAO("roleDAO");
			//invoking the DAO method and stores the return value
			System.out.println("RoleServiceImpl : deleteRoles"+" before executing deleteRoles");
			iResult=roleDAO.deleteRoles(iRoleId);
			System.out.println("RoleServiceImpl : deleteRoles"+"after executing deleteRoles");
			//committing the transaction
			if(1 == iResult)
				{
				System.out.println("RoleServiceImpl : deleteRoles"+" deleted successfully");
				}
			else
				{
				System.out.println("RoleServiceImpl : deleteRoles"+" delete unsuccessful");
				}
			
			}
		catch(ItawException ItawException)
		{
			//Rollback code
			  try{				
					roleDAO.getTransaction().rollback();	
					}
					catch(HibernateException dbe){
					throw new ItawException("ITAW12501");
					}
//					catch(HibernateException dbe){
//					throw new ItawException("ITAW12502");
//				    }
			throw ItawException;
		}
		/*catch (DAOServiceException daoException)
		{
			throw new ItawException("ITAW1028");
		}*/
		catch(Exception exception)
		{
			//Rollback code
			  try{				
				roleDAO.getTransaction().rollback();	
				}
				catch(HibernateException dbe){
				throw new ItawException("ITAW12503");
				}
//				catch(HibernateException dbe){
//				throw new ItawException("ITAW12504");
//			    }
			throw new ItawException("ITAW1029");
			
		}finally{
			try{
				
				roleDAO.closeSession();
					
				
			}catch(HibernateException dbe){
				throw new ItawException("ITAW11358");
			}
		}
		return iResult;
	}

	/**
	 * Modifies a role and return 1 on success
	 * @param roleVO
	 * @param iRoleId
	 * @return 1 on success
	 * @throws 
	 */
	public int modifyRole(RoleDTO role,int iRoleId)throws ItawException
	{
		//iResult is used to store the return value
		int iResult=0;
		try
		{
			//Begining transaction
			roleDAO.getSession().beginTransaction();
			//roleDAO = (roleDAO)hibernateDAOService.getDAO("roleDAO");
			//invoking the DAO method and stores the return value
			iResult=roleDAO.modifyRole(role,iRoleId);
			System.out.println("RoleServiceImpl : Modify Role "+ "iResult="+iResult);
			//committing the transaction
			if(2 == iResult)
			{
				System.out.println("RoleServiceImpl : Modify Role " + "Role Exists"+role.getStrRoleName());
			}
			else if(1 == iResult)
			{
				System.out.println("RoleServiceImpl : Modify Role "+ "Modify Successful"+iRoleId);
			}
			else
			{
				System.out.println("RoleServiceImpl : Modify Role "+ "Modify Failed"+iRoleId);
			}
			
		}
		catch(ItawException ItawException)
		{
			//Rollback code
			  try{				
					roleDAO.getTransaction().rollback();	
					}
					catch(HibernateException dbe){
					throw new ItawException("ITAW12501");
					}
//					catch(HibernateException dbe){
//					throw new ItawException("ITAW12502");
//				    }
			throw ItawException;
		}
		/*catch (DAOServiceException daoException)
		{
			throw new ItawException("ITAW1030");
		}*/
		catch(Exception exception)
		{
			//Rollback code
			  try{				
					roleDAO.getTransaction().rollback();	
					}
					catch(HibernateException dbe){
					throw new ItawException("ITAW12503");
					}
//					catch(HibernateException dbe){
//					throw new ItawException("ITAW12504");
//				    }
			throw new ItawException("ITAW1031");
			
		}finally{
			try{
				
				roleDAO.closeSession();
					
				
			}catch(HibernateException dbe){
				throw new ItawException("ITAW11359");
			}
		}
		return iResult;
	}
	
	public String getName() {
		// TODO Auto-generated method stub
		return null;
	}

	
	/**
	 * fetch all the privileges
	 * @return list of privileges
	 */
	public List listAllPrivileges() throws ItawException 
	{
		//lstAllPrivileges is used to store the list of Privileges
		List lstAllPrivileges = null;
		try
		{
			//roleDAO = (RoleDAO)hibernateDAOService.getDAO("RoleDAO");
			lstAllPrivileges = roleDAO.listAllPrivileges();
			System.out.println("Privilege Service logger "+"Retrieved Privileges:"+lstAllPrivileges.size());
		}
		catch(ItawException ItawException)
		{
			throw ItawException;
		}
		/*catch (DAOServiceException daoException)
		{
			throw new ItawException("ITAW1063");
		}*/
		catch(Exception exception)
		{
			throw new ItawException("ITAW1064");
			
		}finally{
			try{ 
				
				roleDAO.closeSession();
					
				
			}catch(HibernateException dbe){
				throw new ItawException("ITAW11361");
			}
		}
		return lstAllPrivileges;	
	}
	
	/**
	 * fetch all the role details
	 * @return list of role details
	 */
	public List listRoleDetails(int iRoleId) throws ItawException 
	{
		//lstRoleDetails is used to store the list of RoleDetails
		List lstRoleDetails = null;
		try
		{
			//roleDAO = (RoleDAO)hibernateDAOService.getDAO("RoleDAO");
			lstRoleDetails = roleDAO.listRoleDetails(iRoleId);
		}
		catch(ItawException ITAWException)
		{
			throw ITAWException;
		}
		/*catch (DAOServiceException daoException)
		{
			throw new ItawException("ITAW1061");
		}*/
		catch(Exception exception)
		{
			throw new ItawException("ITAW1062");
			
		}finally{
			try{
				
				roleDAO.closeSession();
					
				
			}catch(HibernateException dbe){
				throw new ItawException("ITAW11360");
			}
		}
		return lstRoleDetails;
	}
	
		


@Override
public List listTreePrivileges() throws ItawException {
	// TODO Auto-generated method stub
	return null;
}


@Override
public List listPrivilegeDetails(int iRoleId) throws ItawException {
	// TODO Auto-generated method stub
	return null;
}


@Override
public int associateNewRolePrivilege(int newPrivilegeIds, int roleId) throws ItawException {
	// TODO Auto-generated method stub
	return 0;
}


@Override
public int disassociateRolePrivilege(int privilegeId, int roleId) throws ItawException {
	// TODO Auto-generated method stub
	return 0;
}


@Override
public List searchPrivileges(String strPrivilegeName) throws ItawException {
	// TODO Auto-generated method stub
	return null;
}


@Override
public List listRolesForCreate() throws ItawException {
	// TODO Auto-generated method stub
	return null;
}


}
